@@ -61,10 +61,10 @@ struct _BusDBusImpl {
     /* a serial number used to generate a unique name of a bus. */
     guint id;
 
-    GMutex *dispatch_lock;
+    GMutex dispatch_lock;
     GList *dispatch_queue;
 
-    GMutex *forward_lock;
+    GMutex forward_lock;
     GList *forward_queue;
 
     /* a list of BusMethodCall to be used to reply when services are
@@ -582,8 +582,8 @@ bus_dbus_impl_init (BusDBusImpl *dbus)
                                          NULL,
                                          (GDestroyNotify) bus_name_service_free);
 
-    dbus->dispatch_lock = g_mutex_new ();
-    dbus->forward_lock = g_mutex_new ();
+    g_mutex_init (&dbus->dispatch_lock);
+    g_mutex_init (&dbus->forward_lock);
 
     /* other members are automatically zero-initialized. */
 }
@@ -633,6 +633,9 @@ bus_dbus_impl_destroy (BusDBusImpl *dbus)
                       (GDestroyNotify) bus_method_call_free);
     dbus->start_service_calls = NULL;
 
+    g_mutex_clear (&dbus->dispatch_lock);
+    g_mutex_clear (&dbus->forward_lock);
+
     /* FIXME destruct _lock and _queue members. */
     IBUS_OBJECT_CLASS(bus_dbus_impl_parent_class)->destroy ((IBusObject *) dbus);
 }
@@ -1711,11 +1714,11 @@ bus_dbus_impl_forward_message_idle_cb (BusDBusImpl   *dbus)
 {
     g_return_val_if_fail (dbus->forward_queue != NULL, FALSE);
 
-    g_mutex_lock (dbus->forward_lock);
+    g_mutex_lock (&dbus->forward_lock);
     BusForwardData *data = (BusForwardData *) dbus->forward_queue->data;
     dbus->forward_queue = g_list_delete_link (dbus->forward_queue, dbus->forward_queue);
     gboolean has_message = (dbus->forward_queue != NULL);
-    g_mutex_unlock (dbus->forward_lock);
+    g_mutex_unlock (&dbus->forward_lock);
 
     do {
         const gchar *destination = g_dbus_message_get_destination (data->message);
@@ -1782,10 +1785,10 @@ bus_dbus_impl_forward_message (BusDBusImpl   *dbus,
     data->message = g_object_ref (message);
     data->sender_connection = g_object_ref (connection);
 
-    g_mutex_lock (dbus->forward_lock);
+    g_mutex_lock (&dbus->forward_lock);
     gboolean is_running = (dbus->forward_queue != NULL);
     dbus->forward_queue = g_list_append (dbus->forward_queue, data);
-    g_mutex_unlock (dbus->forward_lock);
+    g_mutex_unlock (&dbus->forward_lock);
 
     if (!is_running) {
         g_idle_add_full (G_PRIORITY_DEFAULT,
@@ -1832,20 +1835,20 @@ bus_dbus_impl_dispatch_message_by_rule_idle_cb (BusDBusImpl *dbus)
 
     if (G_UNLIKELY (IBUS_OBJECT_DESTROYED (dbus))) {
         /* dbus was destryed */
-        g_mutex_lock (dbus->dispatch_lock);
+        g_mutex_lock (&dbus->dispatch_lock);
         g_list_free_full (dbus->dispatch_queue,
                           (GDestroyNotify) bus_dispatch_data_free);
         dbus->dispatch_queue = NULL;
-        g_mutex_unlock (dbus->dispatch_lock);
+        g_mutex_unlock (&dbus->dispatch_lock);
         return FALSE; /* return FALSE to prevent this callback to be called again. */
     }
 
     /* remove fist node */
-    g_mutex_lock (dbus->dispatch_lock);
+    g_mutex_lock (&dbus->dispatch_lock);
     BusDispatchData *data = (BusDispatchData *) dbus->dispatch_queue->data;
     dbus->dispatch_queue = g_list_delete_link (dbus->dispatch_queue, dbus->dispatch_queue);
     gboolean has_message = (dbus->dispatch_queue != NULL);
-    g_mutex_unlock (dbus->dispatch_lock);
+    g_mutex_unlock (&dbus->dispatch_lock);
 
     GList *link = NULL;
     GList *recipients = NULL;
@@ -1899,11 +1902,11 @@ bus_dbus_impl_dispatch_message_by_rule (BusDBusImpl     *dbus,
     g_object_set_qdata ((GObject *) message, dispatched_quark, GINT_TO_POINTER (1));
 
     /* append dispatch data into the queue, and start idle task if necessary */
-    g_mutex_lock (dbus->dispatch_lock);
+    g_mutex_lock (&dbus->dispatch_lock);
     gboolean is_running = (dbus->dispatch_queue != NULL);
     dbus->dispatch_queue = g_list_append (dbus->dispatch_queue,
                     bus_dispatch_data_new (message, skip_connection));
-    g_mutex_unlock (dbus->dispatch_lock);
+    g_mutex_unlock (&dbus->dispatch_lock);
     if (!is_running) {
         g_idle_add_full (G_PRIORITY_DEFAULT,
                          (GSourceFunc) bus_dbus_impl_dispatch_message_by_rule_idle_cb,
