@@ -34,8 +34,7 @@ ibus_xml_free (XMLNode *node)
 
     g_strfreev (node->attributes);
 
-    g_list_foreach (node->sub_nodes, (GFunc) ibus_xml_free, NULL);
-    g_list_free (node->sub_nodes);
+    g_list_free_full (node->sub_nodes, (GDestroyNotify) ibus_xml_free);
 
     g_slice_free (XMLNode, node);
 }
@@ -115,7 +114,7 @@ _end_element_cb (GMarkupParseContext *context,
     XMLNode *p = (XMLNode *) g_markup_parse_context_pop (context);
 
     if (p->text && p->sub_nodes) {
-        g_debug ("Error");
+        g_warning ("Error");
     }
 
     if (p->text == NULL && p->sub_nodes == NULL) {
@@ -176,7 +175,7 @@ static GMarkupParser parser = {
 XMLNode *
 ibus_xml_parse_file (const gchar *filename)
 {
-    gboolean retval;
+    gboolean retval = FALSE;
     GError *error = NULL;
     FILE *pf = fopen (filename, "r");
 
